// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: Editor.java
// Classes: Editor
// Original Author: ics125b spring 1996
// $Id: Editor.java,v 1.17 1996/09/17 01:19:13 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import java.lang.*;
import java.util.*;


/** This class provides an editor frame for manipulating graphical
 * documents. The editor is the central class of the graph editing
 * framework, but it does not contain very much code. It can be this
 * small because all the net-level models, graphical objects, layers,
 * editor modes, editor actions, and supporting dialogs and frames are
 * implemented in their own classes. The framework defines abstract
 * and example classes for each of those class categories.<p>
 *
 * An editor presents a tree of Layer's. Normally layers contain
 * DiagramElement's. Some DiagramElement's are linked to
 * NetPrimitive's. When DiagramElement's are selected the Editor holds
 * a Selection object. The behavior of the editor is determined by its
 * current Mode. The Editor acts as a shell for executing Action's
 * that modify the document or the Editor itself. <p>
 *
 * A major goal of the graph editing framework is to make it easy to
 * extend the framework for application to a specific domain. It is
 * very important that new functionality can be added without
 * modifying what is already there. The fairly small size of the
 * editor is a good indicator that it is not a bottleneck for
 * enhancing the framework. <p>
 *
 * @see Layer
 * @see DiagramElement
 * @see NetPrimitive
 * @see Selection
 * @see Mode
 * @see Action
 */

public class Editor extends Frame implements Observer {
  
  /** The user interface mode that the editor is currently
   * in. Generally modes that the user has to think about are a bad
   * idea. But even in a very easy to use editor there are plenty of
   * "spring-loaded" modes that change the way the system interperts
   * input. For example, when placing a new node, the editor is in
   * ModePlace, and when dragging a handle of an object the editor is
   * in ModeModify. In each case moving or dragging the mouse has a
   * different effect from what happens when the editor is in its
   * default mode (usually ModeSelect)
   *
   * @see ModeModify
   * @see ModeSelect
   * @see ModePlace
   */
  private Mode _curMode;	

  /** This points to the document object that the user is working
   * on. At this point the framework does not have a very strong 
   * concept of document and there is no class Document. For now the
   * meaning of this pointer is in the hands of the person applying 
   * this framework to an application. */
  private Object _document;

  /** _selections holds the SelectionMultiple object that describes
   * all the selection objects for what the user currently has 
   * selected */
  private SelectionMultiple _selections = new SelectionMultiple();

  /** Used to remember what NetNode is under the mouse. Used in event
   * dispatching to pass some events to the NetNode under the mouse. */
  private NetNode _currentNode;

  /** This holds the LayerComposite being displayed in the editor */
  private LayerComposite _currentView;

  /** List of net-level nodes in the connected graph model underlying
   * the diagram being shown in this editor. Multiple editors should
   * be able to modify the same net-level model and the changes should
   * be visible in all editors. This feature is not currently
   * demonstrated by class Example
   *
   * @see Example
   */
  private NetList _net;
  
  
  /** default graphical attributes for newly created objects */
  private Hashtable _graphAttrs;

  /** refers to an object that defines the editors menus and maps menu
   * events into Actions */
  protected EditorMenus menu;

  /** Each Editor has a RedrawManager that executes in a separate
   * thread of control to update damaged regions of the display. */
  private RedrawManager _redrawer;
  
  /** default size of the editor window */
  public final int DEFAULT_WIDTH = 300;
  public final int DEFAULT_HEIGHT = 200;

  /** Construct an editor on the given net-list with no predefined
   * diagrams. I don't think that this is ever called. */
  // public Editor(NetList n) {
  //     this(n.name(), 300, 200, null, n);
  // }

  /** Construct a new editor that is a clone of the gien editor */
  public Editor(Editor ed) {
    Panel sub_panel;

    this.setTitle("Graph Editor");
    this.resize(400, 300);

    _net = ed.net();
    _net.addObserver(this);

    _redrawer = new RedrawManager(this);

    _currentView = ed.view();

    _currentView.addObserver(this);
    _graphAttrs = new Hashtable();
    setGraphicAttribute("fontName", "Times Roman");
    setGraphicAttribute("fontSize", new Integer(10));
    setGraphicAttribute("alignment", "Left");
    setGraphicAttribute("LineColor", Color.blue);
    setGraphicAttribute("FillColor", Color.white);

    /* Now set up the menu items */
    menu = new EditorMenus(this);
    setMenuBar(menu.menuBar);
    mode(new ModeSelect(this));
    Globals.curEditor(this);
  }

  /** Clone the receiving editor. Called from ActionSpawn. SUbclasses
   * of Editor should override this method. */
  public Object clone() {
    Editor ed = new Editor(this);
    //              ^^^^^^
    // subclasses will redefine clone to use a different class name
    return ed;
  }

  /** Construct a new Editor to edit the given NetList */
  public Editor(String title, int w, int h, NetList n) {
    Panel sub_panel;

    this.setTitle(title);
    this.resize(w, h);

    _net = n;
    _net.addObserver(this);

    _redrawer = new RedrawManager(this);

    _currentView = new LayerComposite();
    _currentView.addSublayer(new LayerGrid());
    // the following line is an example of another "grid"
    // _currentView.addSublayer(new LayerPolar());
    _currentView.addSublayer(new LayerDiagram());

    _currentView.addObserver(this);
    _graphAttrs = new Hashtable();
    setGraphicAttribute("fontName", "Times Roman");
    setGraphicAttribute("fontSize", new Integer(10));
    setGraphicAttribute("alignment", "Left");
    setGraphicAttribute("LineColor", Color.blue);
    setGraphicAttribute("FillColor", Color.white);

    /* Now set up the menu items */
    menu = new EditorMenus(this);
    setMenuBar(menu.menuBar);
    mode(new ModeSelect(this));
    Globals.curEditor(this);    
  }

  /** Editors have a set of graphical attributes just like individual
   * DiagramElement' do. The editor uses these attributes to define
   * the defaults for newly created DiagramElement's.
   * @see DiagramElement
   */
  public Hashtable graphAttrs() { return _graphAttrs; }

  public Object getGraphicAttribute(String k) { return _graphAttrs.get(k); }

  public void setGraphicAttribute(String k, Object v) { _graphAttrs.put(k,v); }

  public void setGraphicAttributes(Hashtable newAttrs) {
    Enumeration cur = newAttrs.keys();
    while (cur.hasMoreElements()) {
      String key = (String) cur.nextElement();
      Object val = newAttrs.get(key);
      setGraphicAttribute(key, val);
    }
  }

  /** Set the current user interface mode to the given Mode instance */
  public void mode(Mode m) {
    _curMode = m;
    m.parent(this);
  }

  /** Paints the graphs nodes by calling draw() on view, selections,
   * and mode.  Whenever their is a change to the screen, this method
   * will eventually be called from the RedrawManager.
   * @see RedrawManager
   **/
  public synchronized void paint(Graphics g) {
    _currentView.draw(g);
    _selections.draw(g);
    if (_curMode != null) _curMode.draw(g);
  }


  /** Calling any one of the following damaged() methods adds a
   * damaged region (one or more rectangles) to this editors 
   * RedrawManager. */
  public void damaged(Rectangle r) { _redrawer.add(r); }
  
  public void damaged(DiagramElement de) {
    if (de != null) _redrawer.add(de.getBoundingBox());
  }
  
  public void damaged(Selection sel) {
    if (sel != null) {
      Rectangle r = sel.getBoundingBox();
      if (r != null) _redrawer.add(r);
    }
  }

  /** Mark the entire visible area of this Editor as
   * damaged. Currently called when a LayerGrid is adjusted. This will
   * be useful for ActionRefresh if I get around to it. Also some
   * Actions may perfer to do this instead of keeping track of all
   * modified objects, but only in cases where most of the visible
   * area is expected to change anyway. */
  public void damageAll() {
    Dimension size = size();
    _redrawer.add(new Rectangle(0,0,size.width,size.height));
  }

  /** Tell my RedrawManager to repair all outstanding damaged regions
   * now. Generally, you should not call this method explicitly. It
   * already gets called periodically from another thread of
   * control. You should call it if you need a very tight user
   * interface feedback loop... */
  public void repairDamage() {
    _redrawer.repairDamage();
  }

  /** Paint anything that can be seen in the given Rectangle. */
  public void paintRect(Rectangle r, Graphics g) {
    // the following line is useful for debugging redraw logic
    // g.drawRect(r.x-1, r.y-1, r.width+2, r.height+2);
    paint(g);
  }

  /** Add the given item to this editors selections */
  public void selectItem(DiagramElement de) {
    damaged(_selections);
    _selections.removeAllElements();
    _selections.addElement(de);
    damaged(de);
  }

  /** Remove the given item from this editors selections */
  public void deselectItem(DiagramElement de) {
    if (_selections.contains(de)) {
      _selections.removeElement(de);
      damaged(de);
    }
  }

  /** Select the given item if it was not already selected, and
   * vis-a-versa */
  public void toggleItem(DiagramElement de) {
    damaged(de);
    if (_selections.contains(de)) _selections.removeElement(de);
    else _selections.addElement(de);
    damaged(de);
  }

  /** Deslect everything that is currently selected */
  public void deselect() {
    damaged(_selections);
    _selections.removeAllElements();
  }

  /** Select a collection of DiagramElement's */
  public void selectItems(Vector items) {
    damaged(_selections);
    _selections.removeAllElements();
    _selections.addAllElements(items);
    damaged(_selections);
  }

  /** Toggle the selection of a collection of DiagramElement's */
  public void toggleItems(Vector items) {
    damaged(_selections);
    _selections.toggleAllElements(items);
    damaged(_selections);
  }

  /** Reply the current selection objects of this editor */
  public SelectionMultiple selection() { return _selections; }



  /** The next several methods process user interface events.  There
   * are some events that the editor handles itself, but most useful
   * work is done by delegating events to the current mode, the
   * DiagramElement under the mouse or the node under the mouse. Menu
   * events are always delegated to my EditorMenus object. This scheme
   * could be improved by using another object that defines the
   * bindings of arbitrary UI events to Actions. <p> Please always
   * keep the number of built-in command and actions to a minimum.
   */
  public boolean keyDown(Event e, int key) {
    boolean ret = false;
    String comp_string1;
    FigText text_object;

    setCurrentNode(e.x, e.y);
    /* keystroke handling for scrolling, currently scrolling is broken */
    /* this does not work because the Graphics instance is only used once */
    /*    Graphics g = getGraphics();
    if (key==Event.LEFT) { g.translate(16,0); ret = true; }
    else if (key==Event.RIGHT) { g.translate(-16,0); ret = true; }
    else if (key==Event.UP) { g.translate(0,-16); ret = true; }
    else if (key==Event.DOWN) { g.translate(0,16); ret = true; }
    */
    if (!ret && _currentNode != null) ret = _currentNode.keyDown(e, key);
    if (!ret ) ret = _selections.keyDown(e, key);
    /* needs-more-work: should use key bindings table */
    if (!ret && (key == 127 || key == 8)) {
      executeAction(new ActionDispose(),e);
      ret = true;
    }
    /* needs-more-work: should allow nudging and tabbing */
    if (!ret) ret = super.keyDown(e,key);
    return ret;
  }

  //  public void updateGraphics() { repairDamage(); }

  /** Returns a collection of all DiagramElements in the layer
   * currently being edited. Not used very much now, but could be
   * useful for many Actions. Now used only by ArcPerspective. */
  public Enumeration diagramElements() { return _currentView.elements(); }

  /** Reply the net-level model being edited by this editor. */
  public NetList net() { return _net; }

  /** Return the LayerComposite that holds the diagram being edited. */
  public LayerComposite view() { return _currentView; }

  /** Add a DiagramElement to the diagram being edited. */
  public void add(DiagramElement de) {
    view().add(de);
    if (de instanceof Perspective) _net.addNode((NetNode)de.owner());
  }

  /** Remove a DiagramElement from the diagram being edited. */
  public void remove(DiagramElement de) {
    view().remove(de);
    if (de instanceof Perspective) _net.removeNode((NetNode)de.owner());
  }

  /** Reply the top DiagramElement in the current layer that contains
   * the given point. This is used in determining what the user
   * clicked on, among other uses. */
  public final DiagramElement pick(Point p) { return pick(p.x, p.y); }

  public DiagramElement pick(int x, int y) { return view().pick(x, y); }

  /** This is dead code. Saving is currently not supported, all the
   * save logic needs a total rewrite. This code should probably go
   * into ActionSave or some such class. */
  public void callEditorFileDialog(boolean is_save_dialog) {
    System.out.println("call file editor");
    System.out.println("Opening files ...");
  }

  /** Pass any Menu events to my EditorMenus object.
   * @see EditorMenus
   */
  protected boolean handleMenuEvent(Event e) {
    Globals.curEditor(this);
    return menu.handleMenuEvent(e);
  }

  /** Currently does nothing. This is wrong because the current mode
   * is lost if you drag out of a window accidently. */
  public boolean mouseExit(Event e, int x, int y) {
    /* Globals.curEditor(null); */
    return false;
  }

  /** When the mouse enters the window, set the Editor's mode to the
   * global next mode. This is the way that free standing Palette's
   * interact with Editors.
   */
  public boolean mouseEnter(Event e, int x, int y) {
    Globals.curEditor(this);
    mode(Globals.mode());
    _curMode.parent(this);
    return false;
  }

  /** Try to delegate all events to the current mode, the NetNode
   * under the mouse, or some other object. If no object that I know
   * about wants to handle it, delegate to super. */
  public boolean mouseUp(Event e, int x, int y) {
    setCurrentNode(e.x, e.y);
    boolean ret = _curMode.mouseUp(e, x, y);
    if (!ret && _currentNode != null) ret = _currentNode.mouseUp(e, x, y);
    if (!ret) ret = super.mouseUp(e, x, y);
    return ret;
  }

  /** Try to delegate all events to the current mode, the NetNode
   * under the mouse, or some other object. If no object that I know
   * about wants to handle it, delegate to super. */
  public boolean mouseDown(Event e, int x, int y) {
    setCurrentNode(e.x, e.y);
    boolean ret = _curMode.mouseDown(e, x, y);
    if (!ret && _currentNode != null) ret = _currentNode.mouseDown(e, x, y);
    if (!ret) ret = super.mouseDown(e, x, y);
    return ret;
  }

  /** Try to delegate all events to the current mode, the NetNode
   * under the mouse, or some other object. If no object that I know
   * about wants to handle it, delegate to super. */
  public boolean mouseMove(Event e,int x,int y) {
    /* mouse moves are too frequent and usually not interesting */
    boolean ret = _curMode.mouseMove(e,x,y);
    if (!ret) ret = super.mouseMove(e,x,y);
    return ret;
  }

  /** Try to delegate all events to the current mode, the NetNode
   * under the mouse, or some other object. If no object that I know
   * about wants to handle it, delegate to super. */
  public boolean mouseDrag(Event e,int x,int y) {
    setCurrentNode(e.x, e.y);
    boolean ret = _curMode.mouseDrag(e,x,y);
    if (!ret && _currentNode != null) ret = _currentNode.mouseDrag(e,x,y);
    if (!ret) ret = super.mouseDrag(e,x,y);
    return ret;
  }


  /** intercepts special events for handling before passing it on to
   * the default event handler. If you modifythis function, make sure
   * you call still call super.handleEvent() otherwise none of the
   * above event handling functions will work. */
  public boolean handleEvent(Event e) {
    // The following line is useful for damage/redraw debugging
    // if (e.metaDown()) { damaged(new Rectangle(e.x, e.y, 100, 100));  }
    switch(e.id) {
    case Event.ACTION_EVENT:
      if (e.target instanceof MenuItem) return handleMenuEvent(e);
      return true;
    case Event.WINDOW_DESTROY:
      close();
      return true;
    }
    boolean ret =  super.handleEvent(e);
    // The following line should not be needed... but there is thread
    // bug in the redraw logic and this line masks it pretty well...
    // Even if it was fixed, having this line allows the redraw thread
    // to be run less often... true?
    repairDamage();
    return ret;
  }

  /** The editor acts as a shell for Actions. This method executes the
   * given action in response to the given event (some Action's look
   * at the Event's that invoke them, even though this is
   * discouraged). The Editor executes the Action in a safe
   * environment so that buggy actions cannot crash the whole
   * Editor. <p>
   * Needs-more-work: Action debugging facilities are needed.
   * Needs-more-work: support for fynamic loading of actions... here?
   * @see Action
   */
  public void executeAction(Action act, Event e) {
    if (act == null) return;
    try { act.doIt(e); }
    catch (java.lang.Throwable ex) {
      System.out.println("While executing " + act.toString() +
			 " on event " + e.toString() +
			 " the following error occured:");
      ex.printStackTrace();
    }
  }


  /** close this editor window */
  public void close() {
    // needs-more-work
    // if (!saved) { ... }
    hide();
    dispose();
  }

  /** find the NetNode under the mouse and store it for later use */
  protected void setCurrentNode(int x, int y) {
    _currentNode = nodeUnderMouse(x,y);
  }


  /** find the NetNode represented by the Perspective under the mouse,
   * if any */
  protected NetNode nodeUnderMouse(int mx, int my) {
    DiagramElement de = pick(mx, my);
    if (de instanceof Perspective) return ((Perspective)de).parentNode;
    return null;
  }

 
  /** get and set document being edited. There are no deep semantics
   * here yet, a "document" is up to you to define. */
  public Object document() { return _document; }
  public void document(Object d) { _document = d; }

  /** process change notifications from objects that the Editor
   * observes. An editor observes its Layer's, which intern observe
   * their DiagramElement's which inturn observe their net-level
   * models. When something changes state the notification is
   * propagated up and eventually cause a damaged region and a
   * redraw. <p>
   * Needs-more-work: This is too complex, indirect, and
   * slow. But the concept is simple.
   */
  public void update(Observable o, Object arg) {
    if (arg instanceof DiagramElement) {
      ((DiagramElement) arg).damagedIn(this);
    }
    if (arg instanceof Vector) {
      String s = (String) ((Vector)arg).elementAt(0);
      Object obj = ((Vector)arg).elementAt(1);
      if (s.equals("remove")) {
	System.out.println("editor got 'remove'");
        DiagramElement de;
	if (obj instanceof DiagramElement) de = (DiagramElement) obj;
	else de = view().perspectiveFor(obj);
	
        if (de != null) remove(de);
	else System.out.println("de is null");
	//de.removeFrom(this);
        deselectItem(de);
      }
      else if (s.equals("refresh")) {
	damageAll();
      }
    }
  } /* end update */


}/* end class Editor */
