// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: FigText.java
// Classes: FigText
// Original Author: ics125b spring 1996
// $Id: FigText.java,v 1.7 1996/09/18 01:24:49 jrobbins Exp $

package uci.graphedit;


import java.awt.*;
import java.util.*;
import java.io.*;


/** This class handles drawing and editing text Fig's in a
 * LayerDiagram. Needs-More-Work: should eventually allow styled text
 * editing, ... someday...  */

public class FigText extends Fig {

  /** font info */
  private Font _font;
  private FontMetrics _fm;
  private int _lineHeight;

  /** Text alignment, see constants below */
  private int _alignment;

  /** The current string to display */
  private String _curText;

  /** Constants to specify text justification. Needs-More-Work: some
  * parts of the code refer to this as "alignment", and that is
  * confusing. */
  public final int JUSTIFY_LEFT = 0;
  public final int JUSTIFY_RIGHT = 1;
  public final int JUSTIFY_CENTER = 2;

  /** Minimum size of a FigText object. */
  public final int MIN_TEXT_WIDTH = 30;

  /** number of handles that FigText provides, currently none */
  public static final int numHandles = 0;
  

  /** Construct a new FigText with the given position, size, color,
   * string, and font. */
  public FigText(int x, int y, int t_width, int t_height,
		Color t_color, String t_font, int fontSize) {
    super(x, y, t_width, t_height, t_color, null, numHandles);
    position(x, y);
    objectWidth = t_width;
    objectHeight = t_height;
    objectLineColor = t_color;
    _font = new Font(t_font, Font.PLAIN, fontSize);
    _alignment = JUSTIFY_CENTER;
    _curText = "";
  }

  /** Construct a new FigText with the given position, size, and attributes. */
  public FigText(int x, int y, int t_width, int t_height, Hashtable gAttrs ) {
    this(x, y, t_width, t_height, Color.blue, "TimesRoman", 10);
    setGraphicAttributes(gAttrs);
  }

  /** Reply a string that indicates how the text is justified: Left,
   * Center, or Right. */
  public String alignmentByName() {
    if (_alignment == JUSTIFY_LEFT) return "Left";
    else if (_alignment == JUSTIFY_CENTER) return "Center";
    else if (_alignment == JUSTIFY_RIGHT) return "Right";
    System.out.println("internal error, unknown text alignment");
    return "Unknown";
  }

  /** Set the text justification given one of these strings: Left,
   * Center, or Right. */
  public void alignmentByName(String alignString) {
    if (alignString.equals("Left")) _alignment = JUSTIFY_LEFT;
    else if (alignString.equals("Center")) _alignment = JUSTIFY_CENTER;
    else if (alignString.equals("Right")) _alignment = JUSTIFY_RIGHT;
    _fm = null;
    measure(); /* really not needed */
  }

  /** Reply the name of the current font for this FigText */
  public String fontName() {
    if (_font == null) return "TimesRoman";
    else return _font.getName();
  }

  /** Set the font for this FigText */
  public void fontName(String fn) {
    _font = new Font(fn, Font.PLAIN, fontSize().intValue());
    _fm = null;
    measure();
  }

  /** Reply the font size of this FigText */
  public Integer fontSize() {
    if (_font == null) return new Integer(10);
    else return new Integer(_font.getSize());
  }

  /** Set the font size for this FigText */
  public void fontSize(Integer fs) {
    _font = new Font(fontName(), Font.PLAIN, fs.intValue());
    _fm = null;
    measure();
  }

  /** Get a graphical attribute by name. FigText defines fontName,
   * fontSize, and alignment. Needs-More-Work: alignment should be 
   * justification. */
  public Object getGraphicAttribute(String k) {
    /* needs-more-work: linear comparison is too slow! */
    /* consider a global or local table of int constants... */
    if (k.equals("fontName")) return fontName();
    else if (k.equals("fontSize")) return fontSize();
    else if (k.equals("alignment")) return alignmentByName();
    else return super.getGraphicAttribute(k);
  }

  /** set a graphical attribute by name. */
  public void setGraphicAttribute(String k, Object v) {
    /* needs-more-work: linear comparison is too slow! */
    /* consider a global or local table of int constants... */
    if (k.equals("fontName")) fontName((String) v);
    else if (k.equals("fontSize")) fontSize((Integer)v);
    else if (k.equals("alignment")) alignmentByName((String) v);
    else super.setGraphicAttribute(k, v);
  }


  /** Compute the overall width and height of the FigText object based
   * on the font, font size, and current text. Needs-More-Work: Right
   * now text objects can get larger when you type more, but they
   * do not get smaller when you backspace.  */  
  protected void  measure() {
    if (_fm == null) {
      _fm = Toolkit.getDefaultToolkit().getFontMetrics(_font);
    }
    int overallWidth = 0;
    int numLines = 1;
    StringTokenizer lines = new StringTokenizer(_curText, "\n", true);
    while (lines.hasMoreTokens()) {
      String curLine = lines.nextToken();
      int lineWidth = _fm.stringWidth(curLine);
      if (curLine.equals("\n")) numLines++;
      else overallWidth = Math.max(lineWidth, overallWidth);
    }
    _lineHeight = _fm.getHeight();
    int overallHeight = _lineHeight * numLines;
    overallWidth = Math.max(MIN_TEXT_WIDTH, overallWidth);
    switch (_alignment) {
    case JUSTIFY_LEFT:
      break;
    case JUSTIFY_CENTER:
      if (objectWidth < overallWidth) {
	position().x -= (overallWidth - objectWidth) / 2;
      }
      break;
    case JUSTIFY_RIGHT:
      if (objectWidth < overallWidth) {
	position().x -= (overallWidth - objectWidth);
      }
      break;
    }
    objectWidth = Math.max(objectWidth, overallWidth);
    objectHeight = Math.max(objectHeight, overallHeight);
  }



  /** Remove the last char from the current string line and return the
   * new string. Called whenever the user hits the backspace
   * key. Needs-More-Work: very slow. This will eventually be replaced
   * by full text editing... if there are any volunteers to do
   * that...*/
  public String deleteLastCharFromString(String string) {
    int string_length = Math.max(string.length() - 1, 0);
    char[] string_char = string.toCharArray();
    String new_string = new String(string_char,0,string_length);
    return new_string;
  }


  /** delete the last char from the current string. Called whenever
   * the user hits the backspace key */
  public void deleteLastChar() {
    _curText = deleteLastCharFromString(_curText);
    measure();
  }




  /** Append a character to the current string */
  public void append(char c) { setText(_curText + c); }

  /** Append a string to the current string */
  public void append(String s) { setText(_curText + s); }

  /** set the current string to be the given string */
  public void setText(String s) { _curText = s; measure(); }

  /** Draw the FigText */
  public void draw(Graphics g) {
    int rx = position().x;
    int ry = position().y;
    int new_x = rx, new_y = ry, new_w, new_h;
    int temp_y;

    g.setFont(_font);
    g.setColor(objectLineColor);
    /* Needs-More-Work: should I have a font color independent of line
     * and fill colors? */
    _fm = g.getFontMetrics(_font);
    temp_y = new_y + _lineHeight;

    StringTokenizer lines = new StringTokenizer(_curText, "\n", true);
    while (lines.hasMoreTokens()) {
      String curLine = lines.nextToken();
      int lineWidth = _fm.stringWidth(curLine);
      switch (_alignment) {
      case JUSTIFY_LEFT:
	break;
      case JUSTIFY_CENTER:
	new_x = rx + ( objectWidth - lineWidth ) / 2;
	break;
      case JUSTIFY_RIGHT:
	new_x = rx + objectWidth  - lineWidth;
	break;
      }
      if (curLine.equals("\n")) temp_y += _lineHeight;
      else g.drawString(curLine, new_x, temp_y);
    }
  }


  /** FigText's do not have handles in this version */
  public int pickHandle(int x, int y) { return -1; }


  /** Needs-More-Work: remove this method, its logic has been moved to
   * SelectionBox. */
  public void drawSelected(Graphics g) { }


  /** When the Editor selects this FigText, that fact should be
   * recorded in a SelectionBox object */
  public Selection selectionObject() { return new SelectionBox(this); }


  /** When the user presses a key when a FigText is selected, that key
   * should be added to the current string, or if the key was
   * backspace, the last character is removed. Needs-More-Work: should
   * also catch arrow keys and mouse clicks for full text
   * editing... someday... */
  public boolean keyDown(Event evt,int key) {
    Globals.curEditor().damaged(this);
    if ((key == 8) || (key == 127)) deleteLastChar();
    else append((char)key);
    Globals.curEditor().damaged(this);
    return true; /* needs-more-work: not all keys are processed... */
  }


} /* end class FigText */

