// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: NetPort.java
// Classes: NetPort
// Original Author: ics125b spring 1996
// $Id: NetPort.java,v 1.3 1996/09/17 00:40:28 jrobbins Exp $

package uci.graphedit;


import java.awt.*;
import java.util.*;
import java.io.*;

/** This class models a port in our underlying connected graph model */

public class NetPort extends NetPrimitive {

  /** Constants useful for determining what side (north, south, east,
   * or west) a port is located on. Maybe this should really be in
   * Perspective. */
  public static final double ang45 = Math.PI / 4;
  public static final double ang135 = 3*Math.PI / 4;
  public static final double ang225 = 5*Math.PI / 4;
  public static final double ang315 = 7*Math.PI / 4;

  /** The NetArc's that are connected to this port. */
  protected Vector arcList;

  /** The NetNode that this port is a part of. */
  protected NetNode parentNode;

  /** Construct a new NetPort with the given parent node and no arcs. */
  public NetPort(NetNode parent) {
    parentNode = parent;
    arcList = new Vector();
  }

  /** Add an arc to the list of arcs connected to this port. Called
   * when the user defines a new arc. Normally, you would not call
   this directly, you would call NetArc#connect(). */
  public void addArc(NetArc arc) { arcList.addElement(arc); }

  /** Remove an arc from the list of arcs connected to this
   * port. Called when the user deletes an arc. Normally, you would
   * not call this directly, you would call NetArc#dispose().*/
  public void removeArc(NetArc arc) { arcList.removeElement(arc); }


  /** Remove this port from the underlying connected graph model and
   * dispose all arcs connected to it. */
  public void dispose() {
    Enumeration arcs = arcList.elements();
    while (arcs.hasMoreElements()) {
      NetArc a = (NetArc) arcs.nextElement();
      a.dispose();
    }
  }

  /** Reply the NetNode that owns this port. */
  public NetNode parentNode() { return parentNode; }

  /** Reply true if this port can legally be connected to the given
   * port. Subclasses may implement this to reflect application
   * specific connection constraints. By default, each port just
   * defers that decision to its parent NetNode. By convention, your
   * implementation should return false if super.canConnectTo() would
   * return false (i.e., deeper subclasses get more constrained). I
   * don't know if that is a good convention. */
  public boolean canConnectTo(NetPort anotherPort) {
    return parentNode.canConnectTo(anotherPort.parentNode(),
				   anotherPort,
				   this);
  }

  /** Application specific hook that is called after a successful
   * connection. */
  public void postConnect(NetPort anotherPort) {
    parentNode.postConnect(anotherPort.parentNode(), this, anotherPort);
  }

  /** Application specific hook that is called after a
   * disconnection. (for now, all disconnections are assumed
   * legal). */
  public void postDisconnect(NetPort anotherPort) {
    parentNode.postDisconnect(anotherPort.parentNode(), this, anotherPort);
  }

  /** Reply the port's sector within the current view.  This version
   * works precisely with square perspectives the angxx constants
   * should be removed and calculated by the port if non-square
   * perspectives will be used.
   * 
   *  Sectors
   *		      \  1   /
   *		       \    /
   *		        \  /
   *		     2   \/   4
   *			 /\
   *		        /  \
   *		       /    \
   *		      /  3   \
   **/

  public int getPortSector(Perspective nodePers) {
    Rectangle nodeBBox = nodePers.getBoundingBox();
    Fig portPaint = nodePers.getPortFig(this);
    Rectangle portBBox = portPaint.getBoundingBox();
    int nbbCenterX = nodeBBox.x + nodeBBox.width / 2;
    int nbbCenterY = nodeBBox.y + nodeBBox.height / 2;
    int pbbCenterX = portBBox.x + portBBox.width / 2;
    int pbbCenterY = portBBox.y + portBBox.height / 2;
        
    if (portPaint != null) {
      int dx = (pbbCenterX - nbbCenterX) * nodeBBox.height;
      int dy = (pbbCenterY - nbbCenterY) * nodeBBox.width;
      double dist = Math.sqrt(dx * dx + dy * dy);
      double ang;
      if (dy > 0) ang = Math.acos(dx / dist);
      else ang = Math.acos(dx / dist) + Math.PI;
      
      if (ang < ang45) return 2;
      else if (ang < ang135) return 1;
      else if (ang < ang225) return 4;
      else if (ang < ang315) return 3;
      else return 2;
    }
    return -1;
  }

} /* end class NetPort */







