// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: Palette.java
// Classes: Palette
// Original Author: ics125b spring 1996
// $Id: Palette.java,v 1.4 1996/09/17 23:26:27 jrobbins Exp $

package uci.graphedit;


import java.awt.*;
import java.util.*;

/** Abstract class with shared behavior for windows that provide a
 * palette of diagram tools and materials. Palettes may be displayed
 * in their own Frame, or they may simply define a Panel that is used
 * as part of a PaletteCompound. */

public abstract class Palette extends Frame {

  /** A collection of ActionComponents displayed in this window */
  private Vector _components;
  
  /** The palette that makes up most of the window */
  private Panel _panel;
  
  /** The size of the grid of of _components */
  private int _rows, _cols;
  
  /** A checkbox that can make some modes sticky */
  private Checkbox _stickyCheckbox;

  /** the most recent button pressed */
  private ActionComponent _lastClicked;

  /** This button should be pressed automatically when the sticky
   * checkbox is unchecked. Usually that is 'select'. */
  private ActionComponent _onUnstick;

  /** Construct a new Palette */
  public Palette() { }

  /** Construct a new Palette with its components arranged in r rows
   * and c columns. */ 
  public Palette(int r, int c) { initialize(r, c); }

  /** Same as previous constructor, but set the title of the palette */
  public Palette(int r, int c, String title) {
    initialize(r, c);
    setUpWindow(title);
  }

  /** Routine to lay out a palette in a Frame. */
  protected void setUpWindow(String title) {
    setTitle(title);
    setBackground(Color.lightGray);
    add("Center", _panel);
    _stickyCheckbox = new Checkbox("Sticky");
    _stickyCheckbox.setState(false);
    add("South", _stickyCheckbox);
    /* maybe add a status pane at the bottom... */
    pack();
  }

  /** Set up the data structures used by Palette's */
  protected void initialize(int r, int c) {
    _rows = r;
    _cols = c;
    _components = defineButtons();
    _panel = definePanel();
    setFont(new Font("Helvetica", Font.PLAIN, 10));
  }

  /** Abstract method to define the components to be shown in a
   * Palette. This should probably be called defineActiveComponents,
   * because Palette's can have more than just buttons.
   *
   * @see PaletteShape#defineButtons 
   * @see PaletteAttr#defineButtons */
  public abstract Vector defineButtons();

  /** Layout the Palette components in a grid */
  public Panel definePanel() {
    Panel p = new Panel();
    _rows = Math.max(_rows, _components.size() / _cols);
    p.setLayout(new GridLayout(_rows,_cols, 0, 0));
    Enumeration cur = _components.elements();
    while (cur.hasMoreElements()) {
      Object obj =  cur.nextElement();
      ActionComponent ac = (ActionComponent) obj;
      p.add(ac.component());
    }
    return p;
  }

  /** Get and set methods */
  protected Vector components() { return _components; }
  public void rows(int r) { _rows = r; }
  public void cols(int c) { _cols = c; }
  public int rows() { return _rows; }
  public int cols() { return _cols; }
  
  /** Handle action events by finding which component was activated
   * and executnig it's associated action.*/
  public boolean action(Event e, Object what) {
    boolean ret = false;
    _lastClicked = null;

    if (e.target == _stickyCheckbox && !_stickyCheckbox.getState()) 
      Globals.mode(null);
    
    Enumeration cur = _components.elements();
    while (cur.hasMoreElements()) {
      ActionComponent ac = (ActionComponent) cur.nextElement();
      if (e.target == ac.component()) {
	Globals.setSticky(_stickyCheckbox.getState());
	ac.doIt(e); 
	// needs-more-work: should go through a bottle-neck method
	if (_stickyCheckbox.getState()) _lastClicked = ac;
	if (!ac.canStick()) _stickyCheckbox.setState(false);
	ret = true;
      }
    }
    highlightStuckButton();
    return ret;
  }

  /** Make the currently stuck button appear to be stuck down. This
   * gives the user the impression that the button is still depressed
   * and that future mouse clicks on the drawing area will make new
   * instances of the same type */
  protected void highlightStuckButton() {
    Enumeration cur = _components.elements();
    while (cur.hasMoreElements()) {
      ActionComponent ac = (ActionComponent) cur.nextElement();
      ac.unhighlight();
    }
    if (_stickyCheckbox.getState() && _lastClicked != null)
      _lastClicked.highlight();
  }

  /** Reply the button that should be automatically pressed when the
   * suer unchecks the sticky checkbox. */
  public void onUnstick(ActionComponent ac) { _onUnstick = ac; }

  /** When the user wants to close the window, close it. */
  public boolean handleEvent(Event e) {
    if (e.id == Event.WINDOW_DESTROY) {
      close();
      return true;
    }
    return super.handleEvent(e);
  }

  /** First hide the window, then dispose of its data structures. */
  public void close() {
    hide();
    dispose();
  }

} /* end class Palette */

